# -*- coding: utf-8 -*-
#
# Copyright (C) 2009-2012 Sebastien Helleu <flashcode@flashtux.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Display content of viedemerde.fr/fmylife.com website.
# (this script requires WeeChat 0.3.7 or newer)
#
# History:
#
# 2012-12-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.3: fix parsing: replace "\r" by space
# 2012-11-12, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.2: use URL transfer in API (for WeeChat >= 0.3.7)
# 2012-01-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.1: make script compatible with Python 3.x
# 2011-03-11, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.0: get python 2.x binary for hook_process (fix problem when
#                  python 3.x is default python version)
# 2011-03-04, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.9: fix memory leak in XML parser
# 2010-04-28, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.8: switch to vdm buffer if already opened
# 2009-10-13, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.7: use "q" to close vdm buffer
# 2009-06-12, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.6: fix bug when vdm buffer is closed: clear old list
# 2009-05-02, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.5: sync with last API changes
# 2009-03-16, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.4: use existing vdm buffer if found (for example after /upgrade)
# 2009-03-15, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.3: do not switch to vdm buffer if there's nothing new to
#                  display, do not display time for each line on vdm buffer
# 2009-03-12, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.2: fix bug with "&quot;" in string
# 2009-03-08, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.1: initial release
#

import weechat, sys, xml.dom.minidom

SCRIPT_NAME    = "vdm"
SCRIPT_AUTHOR  = "Sebastien Helleu <flashcode@flashtux.org>"
SCRIPT_VERSION = "1.3"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Display content of viedemerde.fr/fmylife.com website"

# script options
settings = {
    # language: en or fr
    "lang"             : "en",
    # url for API
    "url"              : "http://api.betacie.com/view/%s?key=readonly&language=%s",
    # auto-switch to buffer when data is ready (when using /vdm command)
    "auto_switch"      : "on",
    # color for # VDM
    "color_number"     : "cyan",
    # display number as prefix for each line?
    "number_as_prefix" : "on",
    # colors for displaying items (one or more colors, separated by ";")
    "colors"           : "default;green;brown",
    # blank line between each VDM
    "blank_line"       : "on",
    # reverse order (most recent entry displayed last)
    "reverse"          : "off",
}

vdm_buffer           = ""
vdm_switch_to_buffer = False
vdm_hook_process     = ""
vdm_key              = ""
vdm_stdout           = ""
vdm_oldlist          = []

if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                    SCRIPT_DESC, "", ""):
    weechat.hook_command("vdm",
                         "Display content of viedemerde.fr/fmylife.com website "
                         "in a buffer",
                         "[[*]key | r | - | + | fr | en]",
                         "key: key for website:\n"
                         "           last: last VDMs\n"
                         "         random: random VDM\n"
                         "            top: top VDMs\n"
                         "           flop: flop VDMs\n"
                         "              #: a VDM number\n"
                         "       category: english: love, money, kids, work, "
                         "health, sex, miscellaneous\n"
                         "                 french: amour, argent, travail, "
                         "sante, sexe, inclassable\n"
                         "     - if key begins with \"*\", then VDM buffer is "
                         "cleared before reading website\n"
                         "     - default key is \"last\"\n"
                         "     - for last/top/flop/category, you can add "
                         "\"/\" + page number (1 is second page)\n"
                         "  r: replay last query\n"
                         "  -: get previous page for last query\n"
                         "  +: get next page for last query\n"
                         " fr: set language to french (content of "
                         "viedemerde.fr)\n"
                         " en: set language to english (content of "
                         "fmylife.com)\n\n"
                         "Examples:\n"
                         "  /vdm last     => display last VDMs\n"
                         "  /vdm last/1   => display second page of last VDMs\n"
                         "  /vdm top      => display top VDMs\n"
                         "  /vdm *random  => clear VDM buffer and display a "
                         "random VDM\n"
                         "  /vdm +        => display next page\n"
                         "  /vdm *+       => clear VDM buffer and display "
                         "next page\n\n"
                         "All arguments for this command can be used as input "
                         "on VDM buffer.",
                         "", "vdm_cmd", "")
    for option, default_value in settings.items():
        if weechat.config_get_plugin(option) == "":
            weechat.config_set_plugin(option, default_value)

def vdm_buffer_set_title():
    """ Set buffer title, using key used by user. """
    global vdm_buffer, vdm_key
    lang = weechat.config_get_plugin("lang")
    weechat.buffer_set(vdm_buffer, "title",
                       SCRIPT_NAME + " " + SCRIPT_VERSION + " [" + lang + "]  |  "
                       + "Keys: last, random, top, flop, #, category "
                       + "(current: '" + vdm_key + "')  |  "
                       + "Get help with /help vdm")

def vdm_display(vdm):
    """ Display VDMs in buffer. """
    global vdm_buffer
    weechat.buffer_set(vdm_buffer, "unread", "1")
    if weechat.config_get_plugin("number_as_prefix") == "on":
        separator = "\t"
    else:
        separator = " > "
    colors = weechat.config_get_plugin("colors").split(";");
    vdm2 = vdm[:]
    if weechat.config_get_plugin("reverse") == "on":
        vdm2.reverse()
    for index, item in enumerate(vdm2):
        item_id = item["id"]
        item_text = item["text"]
        if sys.version_info < (3,):
            # python 2.x: convert unicode to str (in python 3.x, id and text are already strings)
            item_id = item_id.encode("UTF-8")
            item_text = item_text.encode("UTF-8")
        weechat.prnt_date_tags(vdm_buffer,
                               0, "notify_message",
                               "%s%s%s%s%s" %
                               (weechat.color(weechat.config_get_plugin("color_number")),
                                item_id,
                                separator,
                                weechat.color(colors[0]),
                                item_text))
        colors.append(colors.pop(0))
        if index == len(vdm) - 1:
            weechat.prnt(vdm_buffer, "------")
        elif weechat.config_get_plugin("blank_line") == "on":
            weechat.prnt(vdm_buffer, "")

def vdm_parse(string):
    """ Parse XML output from HTTP output string. """
    global vdm_buffer, vdm_switch_to_buffer
    vdm = []
    try:
        dom = xml.dom.minidom.parseString(string)
    except:
        weechat.prnt(vdm_buffer,
                     "%sError reading data from website (maybe it's down? "
                     "please try again later)" % weechat.prefix("error"))
        if vdm_switch_to_buffer:
            weechat.buffer_set(vdm_buffer, "display", "1")
    else:
        for node in dom.getElementsByTagName("item"):
            texte = node.getElementsByTagName("text")
            if texte:
                vdm.append({"id": node.getAttribute("id"),
                            "text": texte[0].firstChild.data.replace("\r", " ").replace("\n", " ").replace("&quot;", "\"")})
        dom.unlink()
    return vdm

def vdm_process_cb(data, command, rc, stdout, stderr):
    """ Callback reading HTML data from website. """
    global vdm_stdout, vdm_switch_to_buffer, vdm_hook_process, vdm_key, vdm_oldlist
    if stdout != "":
        vdm_stdout += stdout
    if int(rc) >= 0:
        list = vdm_parse(vdm_stdout)
        if len(list) > 0 and list != vdm_oldlist:
            vdm_buffer_set_title()
            vdm_display(list)
            vdm_oldlist = list[:]
        if vdm_switch_to_buffer:
            weechat.buffer_set(vdm_buffer, "display", "1")
        vdm_switch_to_buffer = False
        vdm_hook_process = ""
    return weechat.WEECHAT_RC_OK

def vdm_buffer_create():
    """ Create VDM buffer. """
    global vdm_buffer
    vdm_buffer = weechat.buffer_search("python", "vdm")
    if vdm_buffer == "":
        vdm_buffer = weechat.buffer_new("vdm",
                                        "vdm_buffer_input", "",
                                        "vdm_buffer_close", "")
    if vdm_buffer != "":
        vdm_buffer_set_title()
        weechat.buffer_set(vdm_buffer, "localvar_set_no_log", "1")
        weechat.buffer_set(vdm_buffer, "time_for_each_line", "0")

def vdm_get(key):
    """ Get some VDMs by launching background process. """
    global vdm_buffer, vdm_hook_process, vdm_key, vdm_stdout, vdm_oldlist
    # open buffer if needed
    if vdm_buffer == "":
        vdm_buffer_create()
    # set language
    if key == "fr" or key == "en":
        weechat.config_set_plugin("lang", key)
        vdm_buffer_set_title()
        return
    # clear buffer
    if key[0] == "*":
        if vdm_buffer != "":
            weechat.buffer_clear(vdm_buffer)
        vdm_oldlist = []
        key = key[1:]
    # previous page
    if key == "-":
        items = vdm_key.split("/", 1)
        if len(items) == 1:
            page = 0
        else:
            page = int(items[1]) - 1
        if page <= 0:
            vdm_key = items[0]
        else:
            vdm_key = "%s/%s" % (items[0], page)
    elif key == "+":
        # next page
        items = vdm_key.split("/", 1)
        if len(items) == 1:
            page = 1
        else:
            page = int(items[1]) + 1
        vdm_key = "%s/%s" % (items[0], page)
    elif key != "r":
        vdm_key = key
    # get data from website, via hook_process
    if vdm_hook_process != "":
        weechat.unhook(vdm_hook_process)
        vdm_hook_process = ""
    vdm_stdout = ""
    url = weechat.config_get_plugin("url") % (vdm_key, weechat.config_get_plugin("lang"))
    vdm_hook_process = weechat.hook_process("url:%s" % url, 10 * 1000, "vdm_process_cb", "")

def vdm_buffer_input(data, buffer, input_data):
    """ Read data from user in VDM buffer. """
    if input_data == "q" or input_data == "Q":
        weechat.buffer_close(buffer)
    else:
        vdm_get(input_data)
    return weechat.WEECHAT_RC_OK

def vdm_buffer_close(data, buffer):
    """ User closed VDM buffer. Oh no, why? """
    global vdm_buffer, vdm_oldlist
    vdm_buffer = ""
    vdm_oldlist = []
    return weechat.WEECHAT_RC_OK

def vdm_cmd(data, buffer, args):
    """ Callback for /vdm command. """
    global vdm_switch_to_buffer
    if weechat.config_get_plugin("auto_switch") == "on":
        vdm_switch_to_buffer = True
    if args != "":
        vdm_get(args)
    else:
        vdm_get("last")
    return weechat.WEECHAT_RC_OK



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016-2019 by Simmo Saan <simmo.saan@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# History:
#
# 2019-06-27, Simmo Saan <simmo.saan@gmail.com>
#   version 1.1: fix completion newlines
# 2018-06-19, Simmo Saan <simmo.saan@gmail.com>
#   version 1.0: add Python 2/3 compatibility
# 2016-11-01, Simmo Saan <simmo.saan@gmail.com>
#   version 0.9: remove ungrouped script replacement
# 2016-06-30, Simmo Saan <simmo.saan@gmail.com>
#   version 0.8: support vulgar fractions and \sqrt
# 2016-06-18, Simmo Saan <simmo.saan@gmail.com>
#   version 0.7: support super-, subscripts and \frac
# 2016-06-17, Simmo Saan <simmo.saan@gmail.com>
#   version 0.6: add subcommands for manual reload/redownload
# 2016-06-15, Simmo Saan <simmo.saan@gmail.com>
#   version 0.5: remove lxml dependency
# 2016-06-15, Simmo Saan <simmo.saan@gmail.com>
#   version 0.4: more options for replacement location fine tuning
# 2016-06-14, Simmo Saan <simmo.saan@gmail.com>
#   version 0.3: options for disabling/enabling replacements by location
# 2016-06-14, Simmo Saan <simmo.saan@gmail.com>
#   version 0.2: automatically load XML from W3 website if not available
# 2016-06-13, Simmo Saan <simmo.saan@gmail.com>
#   version 0.1: initial script
#

"""
Replace LaTeX with unicode representations
"""

from __future__ import print_function

SCRIPT_NAME = "latex_unicode"
SCRIPT_AUTHOR = "Simmo Saan <simmo.saan@gmail.com>"
SCRIPT_VERSION = "1.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Replace LaTeX with unicode representations"

SCRIPT_REPO = "https://github.com/sim642/latex_unicode"

SCRIPT_COMMAND = SCRIPT_NAME

IMPORT_OK = True

try:
	import weechat
except ImportError:
	print("This script must be run under WeeChat.")
	print("Get WeeChat now at: http://www.weechat.org/")
	IMPORT_OK = False

import os
import xml.etree.ElementTree as ET
import re
import sys

PY2 = sys.version_info < (3,)
if PY2:
	chr = unichr
	range = xrange

SETTINGS = {
	"input": (
		"on",
		"replace LaTeX in input display: off, on",
		True),
	"send": (
		"on",
		"replace LaTeX in input sending: off, on",
		True),
	"buffer": (
		"on",
		"replace LaTeX in buffer: off, on",
		True)
}

SETTINGS_PREFIX = "plugins.var.python.{}.".format(SCRIPT_NAME)

hooks = []

xml_path = None
replacements = []
scripts = {
	u"0": (u"⁰", u"₀"),
	u"1": (u"¹", u"₁"),
	u"2": (u"²", u"₂"),
	u"3": (u"³", u"₃"),
	u"4": (u"⁴", u"₄"),
	u"5": (u"⁵", u"₅"),
	u"6": (u"⁶", u"₆"),
	u"7": (u"⁷", u"₇"),
	u"8": (u"⁸", u"₈"),
	u"9": (u"⁹", u"₉"),

	u"A": (u"ᴬ", None),
	u"B": (u"ᴮ", None),
	u"D": (u"ᴰ", None),
	u"E": (u"ᴱ", None),
	u"G": (u"ᴳ", None),
	u"H": (u"ᴴ", None),
	u"I": (u"ᴵ", None),
	u"J": (u"ᴶ", None),
	u"K": (u"ᴷ", None),
	u"L": (u"ᴸ", None),
	u"M": (u"ᴹ", None),
	u"N": (u"ᴺ", None),
	u"O": (u"ᴼ", None),
	u"P": (u"ᴾ", None),
	u"R": (u"ᴿ", None),
	u"T": (u"ᵀ", None),
	u"U": (u"ᵁ", None),
	u"V": (u"ⱽ", None),
	u"W": (u"ᵂ", None),

	u"a": (u"ᵃ", u"ₐ"),
	u"b": (u"ᵇ", None),
	u"c": (u"ᶜ", None),
	u"d": (u"ᵈ", None),
	u"e": (u"ᵉ", u"ₑ"),
	u"f": (u"ᶠ", None),
	u"g": (u"ᵍ", None),
	u"h": (u"ʰ", u"ₕ"),
	u"i": (u"ⁱ", u"ᵢ"),
	u"j": (u"ʲ", u"ⱼ"),
	u"k": (u"ᵏ", u"ₖ"),
	u"l": (u"ˡ", u"ₗ"),
	u"m": (u"ᵐ", u"ₘ"),
	u"n": (u"ⁿ", u"ₙ"),
	u"o": (u"ᵒ", u"ₒ"),
	u"p": (u"ᵖ", u"ₚ"),
	u"r": (u"ʳ", u"ᵣ"),
	u"s": (u"ˢ", u"ₛ"),
	u"t": (u"ᵗ", u"ₜ"),
	u"u": (u"ᵘ", u"ᵤ"),
	u"v": (u"ᵛ", u"ᵥ"),
	u"w": (u"ʷ", None),
	u"x": (u"ˣ", u"ₓ"),
	u"y": (u"ʸ", None),
	u"z": (u"ᶻ", None),

	u"+": (u"⁺", u"₊"),
	u"-": (u"⁻", u"₋"),
	u"=": (u"⁼", u"₌"),
	u"(": (u"⁽", u"₍"),
	u")": (u"⁾", u"₎"),

	u"β": (u"ᵝ", u"ᵦ"),
	u"γ": (u"ᵞ", u"ᵧ"),
	u"δ": (u"ᵟ", None),
	u"θ": (u"ᶿ", None),
	u"ι": (u"ᶥ", None),
	u"ρ": (None, u"ᵨ"),
	u"φ": (u"ᵠ", u"ᵩ"),
	u"χ": (u"ᵡ", u"ᵪ"),
}
# https://en.wikipedia.org/wiki/Number_Forms
vulgar_fractions = {
	(u"1", u"4"): u"¼",
	(u"1", u"2"): u"½",
	(u"3", u"4"): u"¾",
	(u"1", u"4"): u"¼",
	(u"1", u"7"): u"⅐",
	(u"1", u"9"): u"⅑",
	(u"1", u"10"): u"⅒",
	(u"1", u"3"): u"⅓",
	(u"2", u"3"): u"⅔",
	(u"1", u"5"): u"⅕",
	(u"2", u"5"): u"⅖",
	(u"3", u"5"): u"⅗",
	(u"4", u"5"): u"⅘",
	(u"1", u"6"): u"⅙",
	(u"5", u"6"): u"⅚",
	(u"1", u"8"): u"⅛",
	(u"3", u"8"): u"⅜",
	(u"5", u"8"): u"⅝",
	(u"7", u"8"): u"⅞",
	(u"0", u"3"): u"↉",
}

def log(string):
	"""Log script's message to core buffer."""

	weechat.prnt("", "{}: {}".format(SCRIPT_NAME, string))

def error(string):
	"""Log script's error to core buffer."""

	weechat.prnt("", "{}{}: {}".format(weechat.prefix("error"), SCRIPT_NAME, string))

def setup():
	"""Load replacements from available resource."""

	global xml_path
	xml_path = weechat.string_eval_path_home("%h/latex_unicode.xml", "", "", "")

	if os.path.isfile(xml_path):
		setup_from_file()
	else:
		setup_from_url()

def setup_from_url():
	"""Download replacements and store them in weechat home directory."""

	log("downloading XML...")
	weechat.hook_process_hashtable("url:https://www.w3.org/Math/characters/unicode.xml",
		{
			"file_out": xml_path
		},
		30000, "download_cb", "")

def download_cb(data, command, return_code, out, err):
	"""Load downloaded replacements."""

	log("downloaded XML")
	setup_from_file()
	return weechat.WEECHAT_RC_OK

def setup_from_file():
	"""Load replacements from file in weechat home directory."""

	log("loading XML...")
	global replacements

	root = ET.parse(xml_path)
	for character in root.findall("character"):
		dec = character.get("dec")
		if "-" not in dec: # is not a range of characters
			char = chr(int(dec))
			
			ams = character.find("AMS")
			if ams is not None:
				replacements.append((ams.text, char))

			latex = character.find("latex")
			if latex is not None:
				latex = latex.text.strip()
				if latex[0] == "\\": # only add \commands
					replacements.append((latex, char))

	replacements = sorted(replacements, key=lambda replacement: len(replacement[0]), reverse=True) # sort by tex string length descendingly

	log("loaded XML")
	hook_modifiers()

def hook_modifiers():
	"""Update modifier hooks to match settings."""

	# remove existing modifier hooks
	global hooks
	for hook in hooks:
		weechat.unhook(hook)
	hooks = []

	# add hooks according to settings

	input_option = weechat.config_get_plugin("input")
	if weechat.config_string_to_boolean(input_option):
		hooks.append(weechat.hook_modifier("input_text_display", "modifier_cb", ""))

	send_option = weechat.config_get_plugin("send")
	if weechat.config_string_to_boolean(send_option):
		hooks.append(weechat.hook_modifier("input_text_for_buffer", "modifier_cb", ""))

	buffer_option = weechat.config_get_plugin("buffer")
	if weechat.config_string_to_boolean(buffer_option):
		hooks.append(weechat.hook_modifier("weechat_print", "modifier_cb", ""))

def replace_xml_replacements(string):
	"""Apply XML replacements to message."""

	for tex, char in replacements:
		string = string.replace(tex, char)
	return string

def latex_ungroup(string):
	"""Remove grouping curly braces if present."""

	grouped = string.startswith("{") and string.endswith("}")
	if grouped:
		string = string[1:-1]
	return string

def replace_script(string, script):
	"""Regex substitution function for scripts."""

	grouped = string.startswith("{") and string.endswith("}")
	if grouped:
		string = string[1:-1]

	chars = list(string)
	all = True
	for i in range(len(chars)):
		if chars[i] in scripts and scripts[chars[i]][script] is not None:
			chars[i] = scripts[chars[i]][script]
		else:
			all = False
			break

	if all:
		return "".join(chars)
	else:
		return None

def replace_scripts(string):
	"""Apply super- and subscript replacements to message."""

	def replace(match, script):
		replaced = replace_script(match.group(1), script)
		return replaced if replaced is not None else match.group(0)

	string = re.sub(r"\^({[^}]+})", lambda match: replace(match, 0), string, flags=re.UNICODE)
	string = re.sub(r"_({[^}]+})", lambda match: replace(match, 1), string, flags=re.UNICODE)

	def replace_frac(match):
		vulgar_pair = (latex_ungroup(match.group(1)), latex_ungroup(match.group(2)))
		if vulgar_pair in vulgar_fractions:
			return vulgar_fractions[vulgar_pair]

		replaced1 = replace_script(match.group(1), 0)
		replaced2 = replace_script(match.group(2), 1)
		if replaced1 is not None and replaced2 is not None:
			return replaced1 + u"⁄" + replaced2
		else:
			return match.group(0)

	string = re.sub(r"\\frac({[^}]+}|[^{}])({[^}]+}|[^{}])", replace_frac, string, flags=re.UNICODE)

	def replace_sqrt(match):
		under = match.group(2)
		if under is not None:
			under = re.sub(r"(.)", u"\\1\u0305", latex_ungroup(under), flags=re.UNICODE)
		else:
			under = ""

		if match.group(1) is None:
			return u"√" + under

		replaced = replace_script(match.group(1), 0)
		if replaced is not None:
			return replaced + u"√" + under
		else:
			return match.group(0)

	string = re.sub(r"\\sqrt(?:\[([^\]]+)\])?({[^}]+}|[^{}])?", replace_sqrt, string, flags=re.UNICODE)

	return string

def latex_unicode_replace(string):
	"""Apply all latex_unicode replacements."""

	if PY2:
		string = string.decode("utf-8")

	string = replace_xml_replacements(string)
	string = replace_scripts(string)

	if PY2:
		string = string.encode("utf-8")

	return string

def modifier_cb(data, modifier, modifier_data, string):
	"""Handle modifier hooks."""

	return latex_unicode_replace(string)

def command_cb(data, buffer, args):
	"""Handle command hook."""

	args = args.split()

	if len(args) >= 1:
		if args[0] == "reload":
			setup_from_file()
			return weechat.WEECHAT_RC_OK
		elif args[0] == "redownload":
			setup_from_url()
			return weechat.WEECHAT_RC_OK

	error("invalid arguments")
	return weechat.WEECHAT_RC_ERROR

def config_cb(data, option, value):
	"""Handle config hooks (option changes)."""

	option = option[len(SETTINGS_PREFIX):]

	if SETTINGS[option][2]: # if option requires modifier hooks update
		hook_modifiers()

	return weechat.WEECHAT_RC_OK

if __name__ == "__main__" and IMPORT_OK:
	if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
		weechat.hook_command(SCRIPT_COMMAND, SCRIPT_DESC,
"""reload
 || redownload""",
"""    reload: reload replacements from XML file
redownload: redownload replacements XML file and load it""",
"""reload
 || redownload""".replace("\n", ""),
		"command_cb", "")

		for option, value in SETTINGS.items():
			if not weechat.config_is_set_plugin(option):
				weechat.config_set_plugin(option, value[0])

			weechat.config_set_desc_plugin(option, "%s (default: \"%s\")" % (value[1], value[0]))

		weechat.hook_config(SETTINGS_PREFIX + "*", "config_cb", "")

		setup()



#  Project: chanotify
#  Description: A library that call notify-send when message is received
#  on specific server or channel
#  Author: manzerbredes <manzerbredes@gmx.com>
#  License: GPL3
#
#  0.1.0
#  First version, please ask for feature or bugs on my email !

import weechat as weechat

NAME="chanotify"
VERSION="0.1.0"
LICENCE="GPL3"
AUTHOR="manzerbredes"
DESCRIPTION="Call notify-send command when receive a message on a specific server and channel."
HOMEPAGE="http://people.rennes.inria.fr/Loic.Guegan"
CONFIG= {
    "filters": ("*:*", "List of <server>:<channel> separated by comma that will where chanotify will notify. \
                        Note that <server> or <channel> can be * to match every server or channel."),
    "status": ("on", "On/Off chanotify")
}
# Convenient variables
CHANNEL_BYPASS=list() # Contain channels name that will be automatically notified
SERVER_BYPASS=list() # Contain servers name that will be automatically notifier
ALL_BYPASS=False # If true every server and channel will be notified
FILTERS=dict() # Contain list of channels associated with each servers

# Parse filters parameters
def parse_filters(filters):
    global CHANNEL_BYPASS
    global SERVER_BYPASS
    global ALL_BYPASS
    global FILTERS

    elts=filters.split(",")
    for elt in elts:
        server, channel=elt.split(":")
        if server==channel and server=="*":
            ALL_BYPASS=True
        elif server == "*":
            CHANNEL_BYPASS.append(channel)
        elif channel == "*":
            SERVER_BYPASS.append(server)
        else:
            if server in FILTERS:
                FILTERS[server].append(channel)
            else:
                FILTERS[server]=[channel]

def isNotifiable(server,channel):
    global CHANNEL_BYPASS
    global SERVER_BYPASS
    global ALL_BYPASS
    global FILTERS
    global CONFIG

    if CONFIG["status"][0]=="on":
        if (channel in CHANNEL_BYPASS) or (server in SERVER_BYPASS) or ALL_BYPASS:
            return True
        else:
            if server in FILTERS:
                return (channel in FILTERS[server])
    return False

def on_receive(data, signal, signal_data):
    # Fetch server, msg and buffer
    server = signal.split(",")[0]
    msg = weechat.info_get_hashtable("irc_message_parse", {"message": signal_data})
    buffer = weechat.info_get("irc_buffer", "%s,%s" % (server, msg["channel"]))

    # Notify if we get the buffer
    if buffer and isNotifiable(server,msg["channel"]):
        notify_title="On "+msg["channel"]
        notify_msg=msg["nick"]+"> "+msg["text"]
        weechat.hook_process_hashtable("notify-send",
                { "arg1": "-i", "arg2": "weechat",
                  "arg3": "-a", "arg4": "WeeChat",
                  "arg5": notify_title, "arg6": notify_msg},
                20000, "", "")
    return weechat.WEECHAT_RC_OK

# Load the configuration
def update_config(data, option, value):
    global CONFIG
    option=option.split(".")[-1]
    if option != "filters":
        CONFIG[option]=(value,CONFIG[option][1])
    else: # Reset existing filter configuration and set them
        CHANNEL_BYPASS=list()
        SERVER_BYPASS=list()
        ALL_BYPASS=False
        FILTERS=dict()
        parse_filters(CONFIG["filters"][0])

    return weechat.WEECHAT_RC_OK

# Load the script
if __name__ == "__main__":
    # Register plugin
    weechat.register(NAME, AUTHOR, VERSION, LICENCE, DESCRIPTION, "", "UTF-8")

    # Load or set configuration
    for (option,value) in CONFIG.items():
        c = weechat.config_get_plugin(option)
        if len(c) == 0: # Set
            weechat.config_set_plugin(option, value[0])
        else: # Load
            CONFIG[option]=(c,CONFIG[option][1])

    # Watch config changes
    weechat.hook_config("*%s.*"%NAME, "update_config", "")

    # Parse filters
    parse_filters(CONFIG["filters"][0])

    # Watch incoming message
    weechat.hook_signal("*,irc_in2_privmsg", "on_receive", "")



# -*- coding: utf-8 -*-
#
# Copyright (c) 2010-2010 by drubin <drubin at smartcube.co.za>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Allows you to visually see if there are updates to your weechat system

#Versions
# 0.1 drubin     - First release.
#                - Basic functionality to save version history of your config files (only git, bzr)
# 0.2 ShockkPony - Fixed massive weechat startup time caused by initial config loading

SCRIPT_NAME    = "confversion"
SCRIPT_AUTHOR  = "drubin <drubin at smartcube.co.za>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Stores version controlled history of your configuration files"

import_ok = True
import subprocess
try:
    import weechat
except ImportError:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    import_ok = False


# script options
settings = {
    #Currently supports git and bzr and possibly other that support simple "init" "add *.conf" "commit -m "message" "
    "versioning_method"   : "git",
    "commit_each_change"  : "true",
    "commit_message"      : "Commiting changes",
    #Allows you to not auto commit stuff that relates to these configs
    #, (comma) seperated list of config options
    #The toggle_nicklist script can make this property annoying.
    "auto_commit_ignore"  : "weechat.bar.nicklist.hidden",
}

def shell_in_home(cmd):
    try:
        output = file("/dev/null","w")
        subprocess.Popen(ver_method()+" "+cmd, cwd = weechat_home(),
            stdout= output, stderr=output, shell=True)
    except Exception as e:
        print e

def weechat_home():
    return weechat.info_get ("weechat_dir", "")

def ver_method():
    return weechat.config_get_plugin("versioning_method")

def init_repo():
    #Set up version control (doesn't matter if previously setup for bzr, git)
    shell_in_home("init")
    #Save first import OR on start up if needed.
    commit_cb()

confversion_commit_finish_hook = 0

def commit_cb(data=None, remaning=None):
    global confversion_commit_finish_hook

    # only hook timer if not already hooked
    if confversion_commit_finish_hook == 0:
        confversion_commit_finish_hook = weechat.hook_timer(500, 0, 1, "commit_cb_finish", "")

    return weechat.WEECHAT_RC_OK

def commit_cb_finish(data=None, remaining=None):
    global confversion_commit_finish_hook

    # save before doing commit
    weechat.command("","/save")

    # add all config changes to git
    shell_in_home("add ./*.conf")

    # do the commit
    shell_in_home("commit -m \"%s\"" % weechat.config_get_plugin("commit_message"))

    # set hook back to 0
    confversion_commit_finish_hook = 0

    return weechat.WEECHAT_RC_OK

def conf_update_cb(data, option, value):
    #Commit data if not part of ignore list.
    if weechat.config_get_plugin("commit_each_change") == "true"  and not option in weechat.config_get_plugin("auto_commit_ignore").split(","):
        #Call use pause else /save will be called before the config is actually saved to disc
        #This is kinda hack but better input would be appricated.
        weechat.hook_timer(500, 0, 1, "commit_cb", "")
    return weechat.WEECHAT_RC_OK

def confversion_cmd(data, buffer, args):
    commit_cb()
    return weechat.WEECHAT_RC_OK

if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                    SCRIPT_DESC, "", ""):
    for option, default_value in settings.iteritems():
        if weechat.config_get_plugin(option) == "":
            weechat.config_set_plugin(option, default_value)

    weechat.hook_command("confversion", "Saves configurations to version control", "",
                         "",
                         "", "confversion_cmd", "")
    init_repo()
    hook = weechat.hook_config("*", "conf_update_cb", "")



"""
Copyright (c) 2014-2018 by Vlad Stoica <stoica.vl@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

History
-------
01-05-2014 - Vlad Stoica
Uses a sqlite3 database to store triggers with the replies, has the
ability to ignore channels.
16-08-2015 - Vlad Stoica
Fixed a bug where replies couldn't have `:' in them.
15-02-2018 - Vlad Stoica
Added regex support in triggers, and edited syntax of adding triggers.
The command is now 'add "trigger" "reply"'. Quote marks can be escaped
in triggers or replies by prefixing them with a backslash ('\'). For
example, 'add "\"picture\"" "say \"cheese\"!"' is a valid command and
will reply with 'say "cheese"!' whenever it finds '"picture"' sent.

Bugs: not that i'm aware of.
"""

try:
    import weechat
    import sqlite3
    import re
except ImportError:
    raise ImportError("Failed importing weechat, sqlite3, re")
import os

SCRIPT_NAME = "triggerreply"
SCRIPT_AUTHOR = "Vlad Stoica <stoica.vl@gmail.com>"
SCRIPT_VERSION = "0.3"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Auto replies when someone sends a specified trigger. Now with 100% more regex!"


def print_help():
    """ print the help message """
    weechat.prnt("", """
Triggerreply (trigge.rs) plugin. Automatically replies over specified triggers.
------------
Usage: /triggerreply [list | add | remove | ignore | parse] ARGUMENTS

Commands:
    list   - lists the triggers with replies, and ignored channels
    add    - two arguments: "trigger" and "reply"
           - adds a trigger with the specified reply
           - if '-r' is specified, then the trigger will be parsed as regular expression
    remove - one argument: "trigger"
           - remove a trigger
    ignore - one argument: "server.#channel"
           - ignores a particular channel from a server
    parse  - one argument: "server.#channel"
           - removes a channel from ignored list

Examples:
    /triggerreply add "^[Hh](i|ello|ey)[ .!]*" "Hey there!"
    /triggerreply add "lol" "not funny tho"
    /triggerreply remove lol
    /triggerreply ignore rizon.#help
    /triggerreply parse rizon.#help
""")


def create_db(delete=False):
    """ create the sqlite database """
    if delete:
        os.remove(db_file)
    temp_con = sqlite3.connect(db_file)
    cur = temp_con.cursor()
    cur.execute("CREATE TABLE triggers(id INTEGER PRIMARY KEY, trig VARCHAR, reply VARCHAR);")
    cur.execute("INSERT INTO triggers(trig, reply) VALUES ('trigge.rs', 'Automatic reply');")
    cur.execute("CREATE TABLE banchans(id INTEGER PRIMARY KEY, ignored VARCHAR);")
    cur.execute("INSERT INTO banchans(ignored) VALUES ('rizon.#help');")
    temp_con.commit()
    cur.close()


def search_trig_cb(data, buf, date, tags, displayed, highlight, prefix, message):
    """ function for parsing sent messages """
    if weechat.buffer_get_string(buf, "name") == 'weechat':
        return weechat.WEECHAT_RC_OK

    database = sqlite3.connect(db_file)
    database.text_factory = str
    cursor = database.cursor()

    for row in cursor.execute("SELECT ignored from banchans;"):
        if weechat.buffer_get_string(buf, "name") == row[0]:
            return weechat.WEECHAT_RC_OK

    for row in cursor.execute("SELECT * FROM triggers"):
        try:
            r = re.compile(row[1])
            if r.search(message) is not None:
                weechat.command(buf, "/say %s" % row[2])
        except:
            if row[1] == message:
                weechat.command(buf, "/say %s" % row[2])

    return weechat.WEECHAT_RC_OK


def command_input_callback(data, buffer, argv):
    """ function called when `/triggerreply args' is run """
    database = sqlite3.connect(db_file)
    cursor = database.cursor()
    command = argv.split()

    if len(command) == 0:
        print_help()
        return weechat.WEECHAT_RC_ERROR

    if command[0] == "list":
        weechat.prnt("", "List of triggers with replies:")
        for row in cursor.execute("SELECT * FROM triggers;"):
            weechat.prnt("", str(row[0]) + ". " + row[1] + " -> " + row[2])

        weechat.prnt("", "\nList of ignored channels:")
        for row in cursor.execute("SELECT ignored FROM banchans;"):
            weechat.prnt("", row[0])
    elif command[0] == "add":
        if len(argv) == len(command[0]):
            print_help()
            return weechat.WEECHAT_RC_ERROR

        if argv.count('"') < 4:
            print_help()
            return weechat.WEECHAT_RC_ERROR

        pos = []
        for k, v in enumerate(argv):
            if v == '"' and argv[k - 1] != '\\':
                pos.append(k)

        if len(pos) != 4:
            print_help()
            return weechat.WEECHAT_RC_ERROR

        trigger = argv[pos[0] + 1:pos[1]].replace('\\"', '"')
        reply = argv[pos[2] + 1:pos[3]].replace('\\"', '"')

        try:
            cursor.execute("INSERT INTO triggers(trig, reply) VALUES (?,?)", (trigger, reply,))
        except:
            print_help()
            return weechat.WEECHAT_RC_ERROR

        database.commit()
        weechat.prnt("", "Trigger added successfully!")
    elif command[0] == "remove":
        if len(argv) == len(command[0]):
            print_help()
            return weechat.WEECHAT_RC_ERROR

        try:
            cursor.execute("DELETE FROM triggers WHERE trig = ?", (argv[7:],))
        except:
            print_help()
            return weechat.WEECHAT_RC_ERROR

        database.commit()
        weechat.prnt("", "Trigger successfully removed.")
    elif command[0] == "ignore":
        if len(argv) == len(command[0]):
            print_help()
            return weechat.WEECHAT_RC_ERROR

        try:
            cursor.execute("INSERT INTO banchans(ignored) VALUES (?)", (command[1],))
        except:
            print_help()
            return weechat.WEECHAT_RC_ERROR

        database.commit()
        weechat.prnt("", "Channel successfully added to ignore list!")
    elif command[0] == "parse":
        if len(argv) == len(command[0]):
            print_help()
            return weechat.WEECHAT_RC_ERROR

        try:
            cursor.execute("DELETE FROM banchans WHERE ignored = ?", (command[1],))
        except:
            print_help()
            return weechat.WEECHAT_RC_ERROR

        database.commit()
        weechat.prnt("", "Channel successfully removed from ignored.")

    return weechat.WEECHAT_RC_OK


if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
    db_file = "%s/trigge.rs" % weechat.info_get("weechat_dir", "")

    if not os.path.isfile(db_file):
        create_db()

    weechat.hook_print("", "", "", 1, "search_trig_cb", "")
    weechat.hook_command(SCRIPT_NAME, SCRIPT_DESC, "See `/triggerreply' for more information.", "", "",
                         "command_input_callback", "")



#  Project: lnotify
#  Description: A libnotify script for weechat. Uses
#  subprocess.call to execute notify-send with arguments.
#  Author: kevr <kevr.gtalk@gmail.com>
#  License: GPL3
#
# 0.1.2
# added option to display weechat's icon by tomboy64
#
# 0.1.3
# changed the way that icon to WeeChat notification is specified.
# (No absolute path is needed)
# /usr/bin/notify-send isn't needed anymore.
# (pynotify is handling notifications now)
# changed the way that lnotify works. When using gnome 3, every new
# notification was creating a new notification instance. The way that
# it is now, all WeeChat notifications are in a group (that have the
# WeeChat icon and have WeeChat name).
# Got report that it has better look for KDE users too.
#
# 0.1.4
# change hook_print callback argument type of displayed/highlight
# (WeeChat >= 1.0)
#
# 0.2.0
# - changed entire system to hook_process_hashtable calls to notify-send
# - also changed the configuration option names and methods
# Note: If you want pynotify, refer to the 'notify.py' weechat script
#
# 0.3.0
# - added check to see whether the window has x focus so that notify will
# still fire if the conversation tab is open, but the x window is not.
# Note: This will check whether X is running first and whether xdotool
# is installed. If anybody knows a better way to do this, please let me know.
#
# 0.3.1
# Fix https://github.com/weechat/scripts/issues/114 - where we would get
# notifications for messages that we sent
#
# 0.3.2
# Check if active window is in the ignore_windows_list and skip notification
#
# 0.3.3
# Fix undefined ignore_windows_list.
#
# 0.3.5
# Fix ps call generation in Python 3

from __future__ import unicode_literals

import weechat as weechat
import subprocess
from os import environ, path

lnotify_name = "lnotify"
lnotify_version = "0.3.5"
lnotify_license = "GPL3"

# convenient table checking for bools
true = { "on": True, "off": False }

# declare this here, will be global config() object
# but is initialized in __main__
cfg = None

class config(object):
    def __init__(self):
        # default options for lnotify
        self.opts = {
            "highlight": "on",
            "query": "on",
            "notify_away": "off",
            "icon": "weechat",
        }

        self.init_config()
        self.check_config()

    def init_config(self):
        for opt, value in self.opts.items():
            temp = weechat.config_get_plugin(opt)
            if not len(temp):
                weechat.config_set_plugin(opt, value)

    def check_config(self):
        for opt in self.opts:
            self.opts[opt] = weechat.config_get_plugin(opt)

    def __getitem__(self, key):
        return self.opts[key]

def printc(msg):
    weechat.prnt("", msg)

def handle_msg(data, pbuffer, date, tags, displayed, highlight, prefix, message):
    highlight = bool(highlight) and cfg["highlight"]
    query = true[cfg["query"]]
    notify_away = true[cfg["notify_away"]]
    buffer_type = weechat.buffer_get_string(pbuffer, "localvar_type")
    away = weechat.buffer_get_string(pbuffer, "localvar_away")
    x_focus = False
    window_name = ""
    my_nickname = "nick_" + weechat.buffer_get_string(pbuffer, "localvar_nick")

    # Check if active window is in the ignore_windows_list and skip notification
    if (environ.get('DISPLAY') != None) and path.isfile("/bin/xdotool"):
        cmd_pid="xdotool getactivewindow getwindowpid".split()
        window_pid = subprocess.check_output(cmd_pid).decode("utf-8")
        cmd_name=("ps -ho comm -p %s"%(window_pid)).split()
        window_name = subprocess.check_output(cmd_name).decode("utf-8")
        ignore_windows_list = ["tilda", "gnome-terminal", "xterm"]
        if window_name in ignore_windows_list:
            x_focus = True
            return weechat.WEECHAT_RC_OK

    if pbuffer == weechat.current_buffer() and x_focus:
        return weechat.WEECHAT_RC_OK

    if away and not notify_away:
        return weechat.WEECHAT_RC_OK

    if my_nickname in tags:
        return weechat.WEECHAT_RC_OK

    buffer_name = weechat.buffer_get_string(pbuffer, "short_name")


    if buffer_type == "private" and query:
        notify_user(buffer_name, message)
    elif buffer_type == "channel" and highlight:
        notify_user("{} @ {}".format(prefix, buffer_name), message)

    return weechat.WEECHAT_RC_OK

def process_cb(data, command, return_code, out, err):
    if return_code == weechat.WEECHAT_HOOK_PROCESS_ERROR:
        weechat.prnt("", "Error with command '%s'" % command)
    elif return_code != 0:
        weechat.prnt("", "return_code = %d" % return_code)
        weechat.prnt("", "notify-send has an error")
    return weechat.WEECHAT_RC_OK

def notify_user(origin, message):
    hook = weechat.hook_process_hashtable("notify-send",
        { "arg1": "-i", "arg2": cfg["icon"],
          "arg3": "-a", "arg4": "WeeChat",
          "arg5": origin, "arg6": message },
        20000, "process_cb", "")

    return weechat.WEECHAT_RC_OK

# execute initializations in order
if __name__ == "__main__":
    weechat.register(lnotify_name, "kevr", lnotify_version, lnotify_license,
        "{} - A libnotify script for weechat".format(lnotify_name), "", "")

    cfg = config()
    print_hook = weechat.hook_print("", "", "", 1, "handle_msg", "")



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2019 by nils_2 <weechatter@arcor.de>
#
# a simple spell correction for a "misspelled" word
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# 2019-07-16: nils_2, (freenode.#weechat)
#       1.0 : fix bug when misspelled word don't have a suggestion (rafasc)
#           : fixed typo in /help text
#
# 2019-03-01: nils_2, (freenode.#weechat)
#       0.9 : fix bug with auto popup 'spell_suggestion' item (StarlitGhost)
#
# 2019-02-24: nils_2, (freenode.#weechat)
#      0.8.1: fix ValueError when a misspelled word don't have a suggestion and you move cursor in input_line
#
# 2019-02-08: nils_2, (freenode.#weechat)
#       0.8 : add: make addword function shortkey compatible (ldlework)
#           : -> https://github.com/weechat/weechat/pull/1288 (WeeChat >=2.4)
#           : add: selected-suggestion color
#           : -> https://github.com/weechat/scripts/issues/203
#           : support for "spell" plugin (WeeChat >=2.5)
#           : item "spell_suggest" renamed to "spell_suggestion"
#           : -> https://github.com/weechat/weechat/issues/1299
#
# 2013-10-04: nils_2, (freenode.#weechat)
#       0.7 : add: addword function (idea by Nei)
#           : localvar wasn't removed after /input return
#
# 2013-10-04: nils_2, (freenode.#weechat)
#       0.6 : add: new option replace_mode
#           : add: support of /eval (weechat >= 0.4.2)
#           : fix: typo in help
#
# 2013-06-27: nils_2, (freenode.#weechat)
#       0.5 : fix: bug with root input bar
#
# 2013-02-16: nils_2, (freenode.#weechat)
#       0.4 : bug with empty localvar removed (reported by swimmer)
#
# 2013-01-31: nils_2, (freenode.#weechat)
#       0.3 : using new info "aspell_dict" (weechat >= 0.4.1)
#
# 2013-01-25: nils_2, (freenode.#weechat)
#       0.2 : make script compatible with Python 3.x
#
# 2012-01-13: nils_2, (freenode.#weechat)
#       0.1 : - initial release -
#
# requires: WeeChat version >= 0.4.0
# better use latest stable or devel version : http://www.weechat.org/download/
#
#
# Development is currently hosted at
# https://github.com/weechatter/weechat-scripts
#

try:
    import weechat, re #sys

except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

SCRIPT_NAME     = "spell_correction"
SCRIPT_AUTHOR   = "nils_2 <weechatter@arcor.de>"
SCRIPT_VERSION  = "1.0"
SCRIPT_LICENSE  = "GPL"
SCRIPT_DESC     = "a spell correction script to use with spell/aspell plugin"

OPTIONS         = { 'auto_pop_up_item'       : ('off','automatic pop-up suggestion item on a misspelled word'),
                    'auto_replace'           : ('on','replaces misspelled word with selected suggestion, automatically. If you use "off" you will have to bind command "/%s replace" to a key' % SCRIPT_NAME),
                    'catch_input_completion' : ('on','will catch the input_complete commands [TAB-key]'),
                    'eat_input_char'         : ('on','will eat the next char you type, after replacing a misspelled word'),
                    'suggest_item'           : ('${white}%S${default}', 'item format (%S = suggestion, %D = dict). Colors are allowed with format "${color}". note: since WeeChat 0.4.2 content is evaluated, see /help eval.'),
                    'hide_single_dict'       : ('on','will hide dict in item if you have a single dict for buffer only'),
                    'complete_near'          : ('0','show suggestions item only if you are n-chars near the misspelled word (0 = off). Using \'replace_mode\' cursor has to be n-chars near misspelled word to cycle through suggestions.'),
                    'replace_mode'           : ('off','misspelled word will be replaced directly by suggestions. Use option \'complete_near\' to specify range and item \'spell_suggestion\' to show possible suggestions.'),
                  }

Hooks = {'catch_input_completion': '', 'catch_input_return': ''}
regex_color=re.compile('\$\{([^\{\}]+)\}')
regex_optional_tags=re.compile('%\{[^\{\}]+\}')
multiline_input = 0
plugin_name = "spell"      # WeeChat >= 2.5
old_plugin_name = "aspell"   # WeeChat < 2.5
# ================================[ weechat options & description ]===============================
def init_options():
    for option,value in OPTIONS.items():
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, value[0])
            OPTIONS[option] = value[0]
        else:
            OPTIONS[option] = weechat.config_get_plugin(option)
        weechat.config_set_desc_plugin(option, '%s (default: "%s")' % (value[1], value[0]))

def toggle_refresh(pointer, name, value):
    global OPTIONS
    option = name[len('plugins.var.python.' + SCRIPT_NAME + '.'):]        # get optionname
    OPTIONS[option] = value                                               # save new value

    if OPTIONS['catch_input_completion'].lower() == "off":
        if Hooks['catch_input_completion']:
            weechat.unhook(Hooks['catch_input_completion'])
            Hooks['catch_input_completion'] = ''
            weechat.unhook(Hooks['catch_input_return'])
            Hooks['catch_input_return'] = ''
    elif OPTIONS['catch_input_completion'].lower() == "on":
        if not Hooks['catch_input_completion']:
            Hooks['catch_input_completion'] = weechat.hook_command_run('/input complete*', 'input_complete_cb', '')
            Hooks['catch_input_return'] = weechat.hook_command_run('/input return', 'input_return_cb', '')

    return weechat.WEECHAT_RC_OK

# ================================[ hooks() ]===============================
# called from command and when TAB is pressed
def auto_suggest_cmd_cb(data, buffer, args):

    arguments = args.split(' ')

    input_line = weechat.buffer_get_string(buffer, 'input')
    weechat.buffer_set(buffer, 'localvar_set_spell_correction_suggest_input_line', '%s' % input_line)

    if args.lower() == 'replace':
        replace_misspelled_word(buffer)
        return weechat.WEECHAT_RC_OK

#    if not weechat.buffer_get_string(buffer,'localvar_spell_correction_suggest_item'):
#        return weechat.WEECHAT_RC_OK

    tab_complete,position,aspell_suggest_item = get_position_and_suggest_item(buffer)
    if not position:
        position = -1

    if arguments[0].lower() == 'addword' and len(arguments) >= 2:
        found_dicts = get_aspell_dict_for(buffer)
        if len(found_dicts.split(",")) == 1 and len(arguments) == 2:
            word = arguments[1]
            weechat.command("","/%s addword %s" % (plugin_name,word) )
        elif arguments[1] in found_dicts.split(",") and len(arguments) == 3:
            word = arguments[2]
            weechat.command("","/%s addword %s %s" % (plugin_name,arguments[1],word))

    # get localvar for misspelled_word and suggestions from buffer or return
    localvar_aspell_suggest = get_localvar_aspell_suggest(buffer)
    if not localvar_aspell_suggest:
        return weechat.WEECHAT_RC_OK

    if arguments[0].lower() == 'addword' and len(arguments) == 1:
        found_dicts = get_aspell_dict_for(buffer)
        if not ":" in localvar_aspell_suggest and not "," in found_dicts:
            weechat.command("","/%s addword %s" % (plugin_name,localvar_aspell_suggest))
        else:
            misspelled_word,aspell_suggestions = localvar_aspell_suggest.split(':')
            weechat.command("","/%s addword %s" % (plugin_name,misspelled_word))
        return weechat.WEECHAT_RC_OK

    if not ":" in localvar_aspell_suggest:
        return weechat.WEECHAT_RC_OK

    misspelled_word,aspell_suggestions = localvar_aspell_suggest.split(':')

    aspell_suggestions = aspell_suggestions.replace('/',',')
    aspell_suggestion_list = aspell_suggestions.split(',')
    if len(aspell_suggestion_list) == 0:
        position = -1
        weechat.bar_item_update('spell_correction')
        weechat.bar_item_update('spell_suggestion')
        return weechat.WEECHAT_RC_OK

    # append an empty entry to suggestions to quit without changes.
    if OPTIONS['auto_replace'].lower() == "on":
        aspell_suggestion_list.append('')

    position = int(position)
    # cycle backwards through suggestions
    if args == '/input complete_previous' or args == 'previous':
        # position <= -1? go to last suggestion
        if position <= -1:
            position = len(aspell_suggestion_list)-1
        position -= 1
    # cycle forward through suggestions
    else:
        if position >= len(aspell_suggestion_list)-1:
            position = 0
        else:
            position += 1

    # 2 = TAB or command is called
    weechat.buffer_set(buffer, 'localvar_set_spell_correction_suggest_item', '%s:%s:%s' % ('2',str(position),aspell_suggestion_list[position]))

    weechat.bar_item_update('spell_correction')
    return weechat.WEECHAT_RC_OK


# spell_correction_suggest_item:
def show_spell_correction_item_cb (data, item, window):

    # check for root input bar!
    if not window:
        window = weechat.current_window()
#        weechat.buffer_get_string(buffer,'localvar_spell_correction_suggest_item'):

    buffer = weechat.window_get_pointer(window,"buffer")
    if buffer == '':
        return ''

    tab_complete,position,aspell_suggest_item = get_position_and_suggest_item(buffer)
    if not position or not aspell_suggest_item:
        return ''

    config_spell_suggest_item = weechat.config_get_plugin('suggest_item')

    dict_found = search_dict(buffer,position)

    if dict_found:
        if config_spell_suggest_item:
            show_item = config_spell_suggest_item.replace('%S',aspell_suggest_item)
            show_item = show_item.replace('%D',dict_found)
            show_item = substitute_colors(show_item)
            return '%s' % (show_item)
        else:
            return aspell_suggest_item
    else:
        if config_spell_suggest_item:
            show_item = config_spell_suggest_item.replace('%S',aspell_suggest_item)
            if weechat.config_get_plugin('hide_single_dict').lower() == 'off':
                show_item = show_item.replace('%D',get_aspell_dict_for(buffer))
            else:
                show_item = show_item.replace('%D','').rstrip()

            show_item = substitute_colors(show_item)
            return '%s' % (show_item)
    return aspell_suggest_item

# if a suggestion is selected and you edit input line, then replace misspelled word!
def input_text_changed_cb(data, signal, signal_data):
    global multiline_input

    if multiline_input == '1':
        return weechat.WEECHAT_RC_OK

    buffer = signal_data
    if not buffer:
        return weechat.WEECHAT_RC_OK

    tab_complete,position,aspell_suggest_item = get_position_and_suggest_item(buffer)
    if not position or not aspell_suggest_item:
        cursor_pos = weechat.buffer_get_integer(buffer,'input_pos')
        if get_localvar_aspell_suggest(buffer) and cursor_pos >0: # save cursor position of misspelled word
            weechat.buffer_set(buffer, 'localvar_set_current_cursor_pos', '%s' % cursor_pos)
        else:
            saved_cursor_pos = weechat.buffer_get_string(buffer, 'localvar_current_cursor_pos')
            if saved_cursor_pos != '':
                if int(cursor_pos) > int(saved_cursor_pos) + int(OPTIONS['complete_near']) + 3: # +3 to be sure!
                    delete_localvar_replace_mode(buffer)
        return weechat.WEECHAT_RC_OK

    # 1 = cursor etc., 2 = TAB, 3 = replace_mode
    if tab_complete != '0':
        if not aspell_suggest_item:
            aspell_suggest_item = ''
        weechat.buffer_set(buffer, 'localvar_set_spell_correction_suggest_item', '%s:%s:%s' % ('0',position,aspell_suggest_item))
        return weechat.WEECHAT_RC_OK

    if OPTIONS['auto_replace'].lower() == "on":
        replace_misspelled_word(buffer)
        return weechat.WEECHAT_RC_OK

#    weechat.buffer_set(buffer, 'localvar_set_spell_correction_suggest_item', '%s:%s:' % ('0','-1'))
    weechat.bar_item_update('spell_correction')
    weechat.bar_item_update('spell_suggestion')
    return weechat.WEECHAT_RC_OK

# also remove localvar_suggest_item
def replace_misspelled_word(buffer):
    input_line = weechat.buffer_get_string(buffer, 'localvar_spell_correction_suggest_input_line')
    if not input_line:
        # remove spell_correction item
        weechat.buffer_set(buffer, 'localvar_del_spell_correction_suggest_item', '')
        weechat.bar_item_update('spell_correction')
        weechat.bar_item_update('spell_suggestion')
        return
    if OPTIONS['eat_input_char'].lower() == 'off' or input_line == '':
        input_pos = weechat.buffer_get_integer(buffer,'input_pos')
        # check cursor position
        if len(input_line) < int(input_pos) or input_line[int(input_pos)-1] == ' ' or input_line == '':
            input_line = weechat.buffer_get_string(buffer, 'input')

    weechat.buffer_set(buffer, 'localvar_del_spell_correction_suggest_input_line', '')

    localvar_aspell_suggest = get_localvar_aspell_suggest(buffer)

    # localvar_aspell_suggest = word,word2/wort,wort2
    if localvar_aspell_suggest:
        misspelled_word,aspell_suggestions = localvar_aspell_suggest.split(':')
        aspell_suggestions = aspell_suggestions.replace('/',',')
        aspell_suggestion_list = aspell_suggestions.split(',')
    else:
        return

    tab_complete,position,aspell_suggest_item = get_position_and_suggest_item(buffer)
    if not position or not aspell_suggest_item:
        return

    position = int(position)

    input_line = input_line.replace(misspelled_word, aspell_suggestion_list[position])
    if input_line[-2:] == '  ':
        input_line = input_line.rstrip()
        input_line = input_line + ' '

    weechat.buffer_set(buffer,'input',input_line)

    # set new cursor position. check if suggestion is longer or smaller than misspelled word
    input_pos = weechat.buffer_get_integer(buffer,'input_pos') + 1
    length_misspelled_word = len(misspelled_word)
    length_suggestion_word = len(aspell_suggestion_list[position])

    if length_misspelled_word < length_suggestion_word:
        difference = length_suggestion_word - length_misspelled_word
        new_position = input_pos + difference + 1
        weechat.buffer_set(buffer,'input_pos',str(new_position))

    weechat.buffer_set(buffer, 'localvar_del_spell_correction_suggest_item', '')
    weechat.bar_item_update('spell_suggestion')
    weechat.bar_item_update('spell_correction')

# ================================[ subroutines ]===============================
# get aspell dict for suggestion
def search_dict(buffer,position):
    localvar_aspell_suggest = get_localvar_aspell_suggest(buffer)
    dicts_found = localvar_aspell_suggest.count("/")
    if not dicts_found:
        return 0

    # aspell.dict.full_name = en_GB,de_DE-neu
    # localvar_dict = en_GB,de_DE-neu
    dictionary = get_aspell_dict_for(buffer)
    if not dictionary:
        return 0
    dictionary_list = dictionary.split(',')
    # more then one dict?
    if len(dictionary_list) > 1:
        undef,aspell_suggestions = localvar_aspell_suggest.split(':')
        dictionary = aspell_suggestions.split('/')
        words = 0
        i = -1
        for a in dictionary:
            i += 1
            words += a.count(',')+1
            if words > int(position):
                break
        return dictionary_list[i]

# format of localvar aspell_suggest (using two dicts):   diehs:die hs,die-hs,dies/dies,Diebs,Viehs
def get_localvar_aspell_suggest(buffer):
    return weechat.buffer_get_string(buffer, 'localvar_%s_suggest' % plugin_name)

def get_aspell_dict_for(buffer):
    # this should never happens, but to be sure. Otherwise WeeChat will crash
    if buffer == '':
        return ''
    if int(version) >= 0x00040100:
        return weechat.info_get("%s_dict" % plugin_name, buffer)

    # this is a "simple" work around and it only works for buffers with given dictionary
    # no fallback for partial name like "aspell.dict.irc". Get your hands on WeeChat 0.4.1
    full_name = weechat.buffer_get_string(buffer,'full_name')
    return weechat.config_string(weechat.config_get('%s.dict.%s' % (plugin_name,weechat.buffer_get_string(buffer,'full_name'))))

def substitute_colors(text):
    if int(version) >= 0x00040200:
        return weechat.string_eval_expression(text,{},{},{})
    # substitute colors in output
    return re.sub(regex_color, lambda match: weechat.color(match.group(1)), text)

def get_position_and_suggest_item(buffer):
    if weechat.buffer_get_string(buffer,'localvar_spell_correction_suggest_item'):
        tab_complete,position,aspell_suggest_item = weechat.buffer_get_string(buffer,'localvar_spell_correction_suggest_item').split(':',2)
        return (tab_complete,position,aspell_suggest_item)
    else:
        return ('', '', '')

def aspell_suggest_cb(data, signal, signal_data):
    buffer = signal_data
    if OPTIONS['replace_mode'].lower() == 'on':
        localvar_aspell_suggest = get_localvar_aspell_suggest(buffer)
        if localvar_aspell_suggest:
            # aspell says, suggested word is also misspelled. check out if we already have a suggestion list and don't use the new misspelled word!
            if weechat.buffer_get_string(buffer,'localvar_inline_suggestions'):
                return weechat.WEECHAT_RC_OK
            if not ":" in localvar_aspell_suggest:
                return weechat.WEECHAT_RC_OK
            misspelled_word,aspell_suggestions = localvar_aspell_suggest.split(':')
            aspell_suggestions = aspell_suggestions.replace('/',',')
            weechat.buffer_set(buffer, 'localvar_set_inline_suggestions', '%s:%s:%s' % ('2','0',aspell_suggestions))
            weechat.bar_item_update('spell_suggestion')
            return weechat.WEECHAT_RC_OK

    if OPTIONS['auto_pop_up_item'].lower() == 'on':
        auto_suggest_cmd_cb('', buffer, '')
        weechat.buffer_set(buffer, 'localvar_del_spell_correction_suggest_input_line', '')
    weechat.bar_item_update('spell_suggestion')
    return weechat.WEECHAT_RC_OK

def get_last_position_of_misspelled_word(misspelled_word, buffer):
    input_pos = weechat.buffer_get_integer(buffer,'input_pos')
    input_line = weechat.buffer_get_string(buffer, 'input')
    x = input_line.rfind(misspelled_word, 0, int(input_pos))
    y = x + len(misspelled_word)
    return x, y, input_pos

# this is a work-around for multiline
def multiline_cb(data, signal, signal_data):
    global multiline_input

    multiline_input = signal_data
#    if multiline_input == '1':
#        buffer = weechat.window_get_pointer(weechat.current_window(),"buffer")
#        input_line = weechat.buffer_get_string(buffer, 'input')
#    else:
#        buffer = weechat.window_get_pointer(weechat.current_window(),"buffer")
#        input_line_bak = weechat.buffer_get_string(buffer, 'input')

#        if input_line != input_line_bak:
#            input_text_changed_cb('','',buffer)

    return weechat.WEECHAT_RC_OK

# ================================[ hook_keys() ]===============================
# TAB key pressed?
def input_complete_cb(data, buffer, command):

    # check if a misspelled word already exists!
    localvar_aspell_suggest = get_localvar_aspell_suggest(buffer)
    if not localvar_aspell_suggest and not weechat.buffer_get_string(buffer,'localvar_inline_replace_mode'):
        return weechat.WEECHAT_RC_OK

    # first [TAB] on a misspelled word in "replace mode"
    if OPTIONS['replace_mode'].lower() == "on" and not weechat.buffer_get_string(buffer,'localvar_inline_replace_mode') and int(OPTIONS['complete_near']) >= 0:
        weechat.buffer_set(buffer, 'localvar_set_inline_replace_mode', '1')

        if not ":" in localvar_aspell_suggest:
            return weechat.WEECHAT_RC_OK
        misspelled_word,aspell_suggestions = localvar_aspell_suggest.split(':')
        begin_last_position, end_last_position, input_pos = get_last_position_of_misspelled_word(misspelled_word, buffer)

        # maybe nick completion?
        if begin_last_position == -1:
            delete_localvar_replace_mode(buffer)
            return weechat.WEECHAT_RC_OK

        if input_pos - end_last_position > int(OPTIONS['complete_near']):
            delete_localvar_replace_mode(buffer)
            return weechat.WEECHAT_RC_OK

        aspell_suggestions = aspell_suggestions.replace('/',',')
        weechat.buffer_set(buffer, 'localvar_set_inline_suggestions', '%s:%s:%s' % ('2','0',aspell_suggestions))
        weechat.buffer_set(buffer, 'localvar_set_save_position_of_word', '%s:%s' % (begin_last_position, end_last_position))
        inline_suggestions = aspell_suggestions.split(',')

        input_line = weechat.buffer_get_string(buffer, 'input')
        input_line = input_line[:begin_last_position] + inline_suggestions[0] + input_line[end_last_position:]

#        input_line = input_line.replace(misspelled_word, inline_suggestions[0])
        word_differ = 0
        if len(misspelled_word) > len(inline_suggestions[0]):
            word_differ = len(misspelled_word) - len(inline_suggestions[0])
        else:
            word_differ = len(inline_suggestions[0]) - len(misspelled_word)
        if input_line[-2:] == '  ':
            input_line = input_line.rstrip()
            input_line = input_line + ' '

        weechat.buffer_set(buffer,'input',input_line)
        input_pos = int(input_pos) + word_differ
        weechat.buffer_set(buffer,'input_pos',str(input_pos))
        weechat.bar_item_update('spell_suggestion')
        return weechat.WEECHAT_RC_OK

    # after first [TAB] on a misspelled word in "replace mode"
    if OPTIONS['replace_mode'].lower() == "on" and weechat.buffer_get_string(buffer,'localvar_inline_replace_mode') == "1" and int(OPTIONS['complete_near']) >= 0:
        if not ":" in weechat.buffer_get_string(buffer,'localvar_inline_suggestions'):
            return weechat.WEECHAT_RC_OK
        tab_complete,position,aspell_suggest_items = weechat.buffer_get_string(buffer,'localvar_inline_suggestions').split(':',2)

        if not position or not aspell_suggest_items:
            weechat.buffer_set(buffer, 'localvar_del_inline_replace_mode', '')
            return weechat.WEECHAT_RC_OK
        inline_suggestions = aspell_suggest_items.split(',')

        position = int(position)
        previous_position = position
        # cycle backwards through suggestions
        if command == '/input complete_previous':
            # position <= -1? go to last suggestion
            if position <= -1:
                position = len(inline_suggestions)-1
            else:
                position -= 1
        # cycle forward through suggestions
        elif command == '/input complete_next':
            if position >= len(inline_suggestions)-1:
                position = 0
            else:
                position += 1

        begin_last_position, end_last_position, input_pos = get_last_position_of_misspelled_word(inline_suggestions[previous_position], buffer)

        if input_pos - end_last_position > int(OPTIONS['complete_near']):
            delete_localvar_replace_mode(buffer)
            return weechat.WEECHAT_RC_OK

        input_line = weechat.buffer_get_string(buffer, 'input')
        input_line = input_line[:begin_last_position] + inline_suggestions[position] + input_line[end_last_position:]
#        input_line = input_line.replace(inline_suggestions[previous_position], inline_suggestions[position])

        word_differ = 0
        if len(inline_suggestions[previous_position]) > len(inline_suggestions[position]):
            word_differ = len(inline_suggestions[previous_position]) - len(inline_suggestions[position])
        else:
            word_differ = len(inline_suggestions[position]) - len(inline_suggestions[previous_position])

        if input_line[-2:] == '  ':
            input_line = input_line.rstrip()
            input_line = input_line + ' '

        weechat.buffer_set(buffer,'input',input_line)
        input_pos = int(input_pos) + word_differ
        weechat.buffer_set(buffer,'input_pos',str(input_pos))

        weechat.buffer_set(buffer, 'localvar_set_inline_suggestions', '%s:%s:%s' % ('2',str(position),aspell_suggest_items))
        weechat.bar_item_update('spell_suggestion')
        return weechat.WEECHAT_RC_OK

    if int(OPTIONS['complete_near']) > 0:
        if not ":" in localvar_aspell_suggest:
            weechat.bar_item_update('spell_suggestion')
            return weechat.WEECHAT_RC_OK
        misspelled_word,aspell_suggestions = localvar_aspell_suggest.split(':')
        begin_last_position, end_last_position, input_pos = get_last_position_of_misspelled_word(misspelled_word, buffer)
        if input_pos - end_last_position > int(OPTIONS['complete_near']):
            weechat.bar_item_update('spell_suggestion')
            return weechat.WEECHAT_RC_OK

    tab_complete,position,aspell_suggest_item = get_position_and_suggest_item(buffer)
    weechat.buffer_set(buffer, 'localvar_set_spell_correction_suggest_item', '%s:%s:%s' % ('2',position,aspell_suggest_item))

    auto_suggest_cmd_cb('', buffer, command)
    weechat.bar_item_update('spell_suggestion')
    return weechat.WEECHAT_RC_OK


def delete_localvar_replace_mode(buffer):
    if OPTIONS['replace_mode'].lower() == "on":
        weechat.buffer_set(buffer, 'localvar_del_inline_replace_mode', '')
        weechat.buffer_set(buffer, 'localvar_del_inline_suggestions', '')
        weechat.buffer_set(buffer, 'localvar_del_save_position_of_word', '')
        weechat.buffer_set(buffer, 'localvar_del_current_cursor_pos', '')
        weechat.bar_item_update('spell_suggestion')

# if a suggestion is selected and you press [RETURN] replace misspelled word!
def input_return_cb(data, signal, signal_data):
    buffer = signal
    tab_complete,position,aspell_suggest_item = get_position_and_suggest_item(buffer)
    if not position or not aspell_suggest_item:
        delete_localvar_replace_mode(buffer)
        return weechat.WEECHAT_RC_OK
    if OPTIONS['auto_replace'].lower() == "on" and aspell_suggest_item:
        replace_misspelled_word(buffer)
    delete_localvar_replace_mode(buffer)
    return weechat.WEECHAT_RC_OK

# /input delete_*
def input_delete_cb(data, signal, signal_data):
    buffer = signal
    delete_localvar_replace_mode(buffer)
    weechat.buffer_set(buffer, 'localvar_del_spell_correction_suggest_item', '')
    weechat.buffer_set(buffer, 'localvar_del_spell_correction_suggest_input_line', '')
    weechat.bar_item_update('spell_correction')
    weechat.bar_item_update('spell_suggestion')
    return weechat.WEECHAT_RC_OK

# /input move_* (cursor position)
def input_move_cb(data, signal, signal_data):
    buffer = signal

    if OPTIONS['replace_mode'].lower() == "on" and weechat.buffer_get_string(buffer,'localvar_inline_replace_mode') == "1":
        delete_localvar_replace_mode(buffer)
        weechat.buffer_set(buffer, 'localvar_del_spell_correction_suggest_item', '')
        return weechat.WEECHAT_RC_OK

    tab_complete,position,aspell_suggest_item = get_position_and_suggest_item(buffer)

    localvar_aspell_suggest = get_localvar_aspell_suggest(buffer)
    if not localvar_aspell_suggest or not ":" in localvar_aspell_suggest:
        return weechat.WEECHAT_RC_OK

    misspelled_word,aspell_suggestions = localvar_aspell_suggest.split(':')

    if not aspell_suggest_item in aspell_suggestions:
        aspell_suggestion_list = aspell_suggestions.split(',',1)
        weechat.buffer_set(buffer, 'localvar_set_spell_correction_suggest_item', '%s:%s:%s' % ('1',0,aspell_suggestion_list[0]))
        weechat.bar_item_update('spell_correction')
        return weechat.WEECHAT_RC_OK

    weechat.buffer_set(buffer, 'localvar_set_spell_correction_suggest_item', '%s:%s:%s' % ('1',position,aspell_suggest_item))
    return weechat.WEECHAT_RC_OK

def show_spell_suggestion_item_cb (data, item, window):
    buffer = weechat.window_get_pointer(window,"buffer")
    if buffer == '':
        return ''

    if OPTIONS['replace_mode'].lower() == "on":
        if not weechat.buffer_get_string(buffer,'localvar_inline_suggestions'):
            return ''
        tab_complete,position,aspell_suggest_items = weechat.buffer_get_string(buffer,'localvar_inline_suggestions').split(':',2)
        localvar_aspell_suggest = "dummy:%s" % aspell_suggest_items
#        return aspell_suggest_items

    else:
        tab_complete,position,aspell_suggest_item = get_position_and_suggest_item(buffer)
        localvar_aspell_suggest = get_localvar_aspell_suggest(buffer)

    # localvar_aspell_suggest = word,word2/wort,wort2
    if localvar_aspell_suggest:
        try:
            misspelled_word,aspell_suggestions = localvar_aspell_suggest.split(':')
        except ValueError:              # maybe no suggestion for misspelled word. then go back
            return ''
        aspell_suggestions_orig = aspell_suggestions
        aspell_suggestions = aspell_suggestions.replace('/',',')
        aspell_suggestion_list = aspell_suggestions.split(',')

        if not position:
            return ''
        if  position == "-1":
            return aspell_suggestions_orig
        if int(position) < len(aspell_suggestion_list):
            reset_color = weechat.color('reset')
            color = weechat.color(weechat.config_color(weechat.config_get("%s.color.misspelled" % plugin_name)))
            new_word = aspell_suggestion_list[int(position)].replace(aspell_suggestion_list[int(position)],'%s%s%s' % (color, aspell_suggestion_list[int(position)], reset_color))
            aspell_suggestion_list[int(position)] = new_word            # replace word with colored word
            aspell_suggestions_orig = ','.join(map(str, aspell_suggestion_list))
    else:
        return ''
    return aspell_suggestions_orig

def window_switch_cb(data, signal, signal_data):
    weechat.bar_item_update('spell_correction')
    return weechat.WEECHAT_RC_OK
def buffer_switch_cb(data, signal, signal_data):
    weechat.bar_item_update('spell_correction')
    return weechat.WEECHAT_RC_OK

# ================================[ check for nick ]===============================
def weechat_nicklist_search_nick(buffer, nick):
    return weechat.nicklist_search_nick(buffer, "", nick)

# ================================[ main ]===============================
if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        version = weechat.info_get("version_number", "") or 0

        if int(version) < 0x00040000:
            weechat.prnt('','%s%s %s' % (weechat.prefix('error'),SCRIPT_NAME,': needs version 0.4.0 or higher'))
            weechat.command('','/wait 1ms /python unload %s' % SCRIPT_NAME)
        if int(version) < 0x02050000:
            plugin_name = old_plugin_name

        SCRIPT_HELP = """addword   : add a word to personal aspell dictionary (does not work with multiple dicts)
previous  : to cycle though previous suggestion
replace   : to replace misspelled word

Quick start:
    You should add script item "spell_correction" to a bar (i suggest using the input_bar).
    On an misspelled word, press TAB to cycle through suggestions. Press any key on suggestion
    to replace misspelled word with current displayed suggestion.
    Also check script options: /fset %(s)s

IMPORTANT:
    "%(p)s.check.suggestions" option has to be set to a value >= 0 (default: -1 (off)).
    "%(p)s.color.misspelled" option is used to highlight current suggestion in "%(p)s_suggestion" item
    Using "%(p)s.check.real_time" the nick-completion will not work. All misspelled words
    in input_line have to be replaced first.
""" %dict(p=plugin_name, s=SCRIPT_NAME)

        weechat.hook_command(SCRIPT_NAME, SCRIPT_DESC, 'addword <word>||previous||replace',
                            SCRIPT_HELP+
                            '',
                            'previous|replace|addword',
                            'auto_suggest_cmd_cb', '')

        init_options()

        weechat.hook_command_run('/input delete_*', 'input_delete_cb', '')
        weechat.hook_command_run('/input move*', 'input_move_cb', '')
        weechat.hook_signal ('input_text_changed', 'input_text_changed_cb', '')
        # multiline workaround
        weechat.hook_signal('input_flow_free', 'multiline_cb', '')

        weechat.hook_signal ('%s_suggest' % plugin_name, 'aspell_suggest_cb', '')

        weechat.hook_signal ('buffer_switch', 'buffer_switch_cb','')
        weechat.hook_signal ('window_switch', 'window_switch_cb','')

        if OPTIONS['catch_input_completion'].lower() == "on":
            Hooks['catch_input_completion'] = weechat.hook_command_run('/input complete*', 'input_complete_cb', '')
            Hooks['catch_input_return'] = weechat.hook_command_run('/input return', 'input_return_cb', '')
        weechat.hook_config('plugins.var.python.' + SCRIPT_NAME + '.*', 'toggle_refresh', '')
        weechat.bar_item_new('spell_correction', 'show_spell_correction_item_cb', '')
        weechat.bar_item_new('spell_suggestion', 'show_spell_suggestion_item_cb', '')



# Copyright (c) 2010 by John Anderson <sontek@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# 2012-11-17, v0.2 (nils_2@freenode.#weechat)
#     use URL transfer in API (for WeeChat >= 0.3.7), update service URL

import weechat

SCRIPT_NAME    = "whatismyip"
SCRIPT_AUTHOR  = "John Anderson <sontek@gmail.com>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Get your current external ip, using whatismyip.com"

process_output = ""

def whatismyip(data, buffer, args):
    global process_output

    url = 'http://automation.whatismyip.com/n09230945.asp'

    process_output = ''
    url_hook_process = weechat.hook_process("url:%s" % url, 30 * 1000, "process_complete", "")
    return weechat.WEECHAT_RC_OK

def process_complete(data, command, rc, stdout, stderr):
    global process_output
    process_output += stdout.strip()

    if len(process_output) > 40:
        weechat.prnt(weechat.current_buffer(), weechat.prefix("error") + 'whatismyip: [%s]' % "Service Unavailable")
        return weechat.WEECHAT_RC_OK
    if int(rc) == 0:
        weechat.prnt(weechat.current_buffer(), '[%s]' % process_output)

    return weechat.WEECHAT_RC_OK

if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                    SCRIPT_DESC, "", ""):
    weechat.hook_command("whatismyip", SCRIPT_DESC, "", "", "", "whatismyip", "")



# -*- coding: utf-8 -*-

"""
xfer_run_command.py - a weechat script to run a command on xfer_ended signal

Settings:
    * plugins.var.python.xfer_run_command.command <string: the command to run>
      e.g. '/exec mailscript.sh someone@foo.bar -s "XFER {status_string}" -b "File: {filename}"'

Author: Michael Kebe <michael.kebe@gmail.com>
License: GPL3
Date: 25 Oct 2016
"""

import_ok = True

try:
  import weechat
except:
  print("You must run this script within Weechat!")
  print("http://www.weechat.org")
  import_ok = False

SCRIPT_NAME = "xfer_run_command"
SCRIPT_AUTHOR = "Michael Kebe <michael.kebe@gmail.com>"
SCRIPT_VERSION = "0.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Runs a command on xfer_ended signal with acces to data (with trigger not possible)"


OPTIONS = {
  "command" : ("", "This command will be run on xfer_ended signal. You can use the following placeholders: {status_string}, {filename}, {local_filename}, {size}, {remote_nick}."),
}


def xfer_ended_signal_cb(data, signal, signal_data):
  weechat.infolist_next(signal_data)

  command_template = weechat.config_get_plugin("command")
  command_string = command_template.format(
    status_string = weechat.infolist_string(signal_data, 'status_string'),
    filename = weechat.infolist_string(signal_data, 'filename'),
    local_filename = weechat.infolist_string(signal_data, 'local_filename'),
    size = weechat.infolist_string(signal_data, 'size'),
    remote_nick = weechat.infolist_string(signal_data, 'remote_nick'),
  )
  
  rc = weechat.command('', command_string)
  if rc != weechat.WEECHAT_RC_OK:
    weechat.prnt('', "xfer_run_command: there was a problem running command: " + command_string)

  return weechat.WEECHAT_RC_OK


def init_config():
  global OPTIONS
  for option, value in OPTIONS.items():
    weechat.config_set_desc_plugin(option, '%s (default: "%s")' % (value[1], value[0]))
    if not weechat.config_is_set_plugin(option):
      weechat.config_set_plugin(option, value[0])


if __name__ == "__main__" and import_ok:
  if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
    init_config()
    weechat.hook_signal('xfer_ended', 'xfer_ended_signal_cb', '')

